use std::env;
use std::process::exit;

use binary_braillie::to_binary_braillie;

mod common;
use common::argument_parsing;

fn main() {
    let args = parse_args();
    if args.value_to_convert.len() <= 0 {
        print!("{}: No input specified!", args.program_name)
    }
    let mut value: Vec<u8> = if !args.hexadecimal {
        Vec::from(args.value_to_convert.as_bytes())
    }
    else {
        Vec::from(hex::decode(args.value_to_convert).unwrap_or_else(
        |_e_| {
            print!("{}: Invalid hex provided", args.program_name);
            exit(128);
        })
    )};

    if args.reverse_characters { value.reverse(); }

    print!("{}", to_binary_braillie(&value, value.len(), args.mode.try_into().unwrap()))
}



fn parse_args() -> argument_parsing::Arguments {
    let mut args = env::args().peekable();
    let mut rvalue = argument_parsing::Arguments {
        hexadecimal: false,
        mode: 0,
        program_name: args.next().unwrap_or(String::from("t2br")),
        reverse_characters: false,
        value_to_convert: String::new(),
    };
    let program_name = &rvalue.program_name;

    while args.peek() != None {
        let arg = args.next().unwrap();
        
        if arg == "--help"
        || arg == "-h"
        || arg == "-?"
        {
            print_help(rvalue.program_name);
            exit(1);
        }
        else if arg == "--hex" {
            rvalue.hexadecimal = true;
        }
        else if arg == "--direction" {
            match argument_parsing::parse_direction(&mut args, &rvalue, arg) {
                Ok(e) => rvalue.mode = e,
                Err(e) => {
                    print!("{}: {}", program_name, e);
                    exit(64);
                }
            }
        }
        else if arg == "--wrap" {
            match argument_parsing::parse_wrap(&mut args, &rvalue, arg) {
                Ok(e) => rvalue.mode = e,
                Err(e) => {
                    print!("{}: {}", program_name, e);
                    exit(65);
                }
            }
        }
        else if arg == "-r"
        || arg == "--reverse" {
            rvalue.reverse_characters = true;
        }
        else {
            if rvalue.value_to_convert.len() > 0 { rvalue.value_to_convert.push(' '); };
            rvalue.value_to_convert.push_str(&arg);
        }
    }

    return rvalue;
}

fn print_help(program_name: String) {
    print!("Usage: {program_name} [options] <input>\n");
    print!("Convert input to binary represented in braillie characters\n\n");
    print!("  --hex               convert hexadecimal to braillie represented binary\n\n");
    print!("  --direction [mode]  direction of bits per character\n");
    print!("                      possible values: row, row-reverse, column, column-reverse\n\n");
    print!("  --wrap [mode]       direction of bits wrapping per character\n");
    print!("                      possible values: normal, reverse\n\n");
    print!("  -r, --reverse       reverse outputted characters\n\n");
    print!("  -h, -?, --help      display this help and exit");
}
